"""IDotMatrix SDK for controlling IDotMatrix devices."""
# pylint: disable=W0246

from dataclasses import dataclass

from bleak import BleakScanner, AdvertisementData, BleakClient

from .idotmatrix_countdown import IDotMatrixCountDown
from .idotmatrix_effects import IDotMatrixEffects
from .idotmatrix_image import IDotMatrixImage
from .idotmatrix_screen import IDotMatrixScreen
from .idotmatrix_time import IDotMatrixTime

DEVICES_NAMES = "IDM-"


@dataclass
class IDMDevice:
    """Class to represent an IDotMatrix device."""

    name: str
    address: str


class IDotMatrix(
    IDotMatrixScreen,
    IDotMatrixTime,
    IDotMatrixCountDown,
    IDotMatrixEffects,
    IDotMatrixImage,
):
    """Class to control the IDotMatrix device."""

    def __init__(self, mac: str):
        """Initialize the IDotMatrix class."""
        self._mac = mac
        self._connection = BleakClient(self._mac)
        super().__init__(self._connection)

    async def connect(self):
        """Connect to the device."""
        await self._connection.connect()

    async def disconnect(self):
        """Disconnect from the device."""
        if self.is_connected:
            await self._connection.disconnect()

    @property
    def is_connected(self):
        """Check if the device is connected."""
        return self._connection and self._connection.is_connected

    @staticmethod
    async def search_devices() -> list[IDMDevice]:
        """Search for IDotMatrix devices."""
        response = await BleakScanner.discover(return_adv=True)
        devices = []
        for _, (device, advertisement) in response.items():
            if (
                isinstance(advertisement, AdvertisementData)
                and advertisement.local_name
                and advertisement.local_name.startswith(DEVICES_NAMES)
            ):
                devices.append(IDMDevice(advertisement.local_name, device.address))

        return devices



"""IDotMatrixTime class for controlling the time on the IDotMatrix device."""
# pylint: disable=W0246

from datetime import datetime

from bleak import BleakClient

from .idotmatrix_base import IDotMatrixBase
from .idotmatrix_color import IDotMatrixColor


class IDotMatrixTime(IDotMatrixBase):
    """Class for controlling the time on the IDotMatrix device."""

    def __init__(self, connection: BleakClient):
        """Initialize the IDotMatrixTime class."""
        super().__init__(connection)

    async def set_time(self, time: datetime):
        """Set the time for the IDotMatrix device."""
        data = bytearray(
            [
                11,
                0,
                1,
                128,
                time.year % 100,
                time.month,
                time.day,
                time.weekday() + 1,
                time.hour,
                time.minute,
                time.second,
            ]
        )

        await self.write(data)

    async def set_time_mode(
        self, mode: int, show_date: bool, mode24: bool, color: IDotMatrixColor
    ):
        """Set the time mode for the IDotMatrix device."""

        if mode not in range(0, 8):
            raise ValueError(f"Invalid mode ({mode}) value.")

        data: bytearray = bytearray(
            [
                8,
                0,
                6,
                1,
                (mode % 8) | (128 if show_date else 0) | (64 if mode24 else 0),
                color.red,
                color.green,
                color.blue,
            ]
        )
        await self.write(data)



"""IDotMatrix Image Control Class."""
# pylint: disable=W0246

from bleak import BleakClient

from .idotmatrix_base import IDotMatrixBase


class IDotMatrixImage(IDotMatrixBase):
    """Class to control the image mode of the IDotMatrix."""

    def __init__(self, connection: BleakClient):
        """Initialize the image mode control class."""
        super().__init__(connection)

    async def set_image_mode(self, mode: int):
        """Set the image mode for the LED matrix."""
        data = bytearray([5, 0, 4, 1, mode])
        await self.write(data)



"""IDotMatrix Effects Control Class."""
# pylint: disable=W0246

from bleak import BleakClient

from .idotmatrix_base import IDotMatrixBase
from .idotmatrix_color import IDotMatrixColor


class IDotMatrixEffects(IDotMatrixBase):
    """Class to control the LED matrix effects of the IDotMatrix."""

    def __init__(self, connection: BleakClient):
        """Initialize the LED matrix effects control class."""
        super().__init__(connection)

    async def set_color(self, color: IDotMatrixColor):
        """Set the color of the LED matrix."""
        data = bytearray(
            [
                7,
                0,
                2,
                2,
                color.red,
                color.green,
                color.blue,
            ]
        )
        await self.write(data)



"""IDotMatrix screen control class."""
# pylint: disable=W0246

from bleak import BleakClient

from .idotmatrix_base import IDotMatrixBase


class IDotMatrixScreen(IDotMatrixBase):
    """Class to control the screen of the IDotMatrix."""

    def __init__(self, connection: BleakClient):
        """Initialize the screen control class."""
        super().__init__(connection)

    async def turn_off(self):
        """Turn off the IDotMatrix screen."""
        data = bytearray([5, 0, 7, 1, 0])
        await self.write(data)

    async def turn_on(self):
        """Turn on the IDotMatrix screen."""
        data = bytearray([5, 0, 7, 1, 1])
        await self.write(data)

    async def flip(self, flip: bool):
        """Flip the IDotMatrix screen."""
        data = bytearray([5, 0, 6, 128, 1 if flip else 0])
        await self.write(data)

    async def set_brightness(self, brightness: int):
        """Set the brightness of the IDotMatrix screen. Minimum is 5, maximum is 100."""
        if brightness not in range(5, 101):
            raise ValueError(f"Invalid brightness ({brightness}) argument.")

        brightness = max(5, min(100, brightness))
        data = bytearray([5, 0, 4, 128, brightness])
        await self.write(data)



"""An IDotMatrix color module."""
# pylint: disable=W0246

from dataclasses import dataclass
from random import randint


@dataclass
class IDotMatrixColor:
    """Class representing a color for the IDotMatrix."""

    red: int
    green: int
    blue: int

    @staticmethod
    def random_color():
        """Generate a random color."""
        return IDotMatrixColor(randint(0, 255), randint(0, 255), randint(0, 255))



"""IDotMatrix countdown timer control class."""
# pylint: disable=W0246

from bleak import BleakClient

from .idotmatrix_base import IDotMatrixBase


class IDotMatrixCountDown(IDotMatrixBase):
    """Class to control the countdown timer of the IDotMatrix."""

    def __init__(self, connection: BleakClient):
        """Initialize the countdown timer control class."""
        super().__init__(connection)

    async def start_countdown(self, minutes: int, seconds: int):
        """Start the countdown timer for the specified number of minutes and seconds."""
        if minutes not in range(0, 256):
            raise ValueError(f"Invalid minutes ({minutes}) argument.")

        if seconds not in range(0, 60):
            raise ValueError(f"Invalid seconds ({seconds}) argument.")

        data = bytearray(
            [
                7,
                0,
                8,
                128,
                1,
                minutes,
                seconds,
            ]
        )
        await self.write(data)

    async def pause_countdown(self):
        """Pause the countdown timer."""
        data = bytearray(
            [
                7,
                0,
                8,
                128,
                2,
                0,
                0,
            ]
        )
        await self.write(data)

    async def resume_countdown(self):
        """Resume the countdown timer."""
        data = bytearray(
            [
                7,
                0,
                8,
                128,
                3,
                0,
                0,
            ]
        )
        await self.write(data)

    async def stop_countdown(self):
        """Stop the countdown timer."""
        data = bytearray(
            [
                7,
                0,
                8,
                128,
                0,
                0,
                0,
            ]
        )
        await self.write(data)



"""IDotMatrix Base Class."""
# pylint: disable=W0246

from asyncio import sleep

from bleak import BleakClient


UUID_WRITE = "0000fa02-0000-1000-8000-00805f9b34fb"
UUID_READ = "0000fa03-0000-1000-8000-00805f9b34fb"


class IDotMatrixBase:
    """Base class for IDotMatrix control classes."""

    def __init__(self, connection: BleakClient):
        """Initialize the base class with a BleakClient connection."""
        self._connection = connection

    async def write(self, data: bytes, response: bool = False):
        """Send data to the device."""
        if not self._connection.is_connected:
            return

        await self._connection.write_gatt_char(UUID_WRITE, data, response)
        await sleep(0.1)

    async def read(self) -> bytes:
        """Read data from the device."""
        if not self._connection.is_connected:
            return bytes()

        data = await self._connection.read_gatt_char(UUID_READ)
        return data
